FusionHUD = LibStub("AceAddon-3.0"):NewAddon("FusionHUD", "AceConsole-3.0", "AceEvent-3.0");
local AceGUI = LibStub("AceGUI-3.0");

local _G = _G;
FusionHUDLastUpdateClock = GetTime();
 
FusionHUDOptions = {
    name = "Fusion HUD",
    handler = FusionHUD,
	childGroups = "tree",
    type = "group",
    args = {
		newBar = {
			name = "New Bar",
			type = "execute",
			desc = "Creates a new HUD bar",
			order = 1,
			width = "normal",
			func = 		function()
							local index = 0;
							table.foreach(FusionHUDBars, function(k,v) if index < v.barIndex then index = v.barIndex end end);
							FusionHUDBars["bar" .. index + 1] = FusionHUDBarPrototype:New(index + 1)
						end,
		},
		deleteBar = {
			name = "Delete Bar",
			order = 2,
			type = "execute",
			desc = "Deletes the last bar created",
			width = "normal",
			confirm = function() return "Are you sure? This cannot be undone" end,
			func =	function()
						FusionHUD:DeleteBar() 
					end	
		},
		lockAll = {
			name = "Unlock All Bars",
			type = "execute",
			desc = "Unlocks all HUD bars for repositioning",
			order = 4,
			width = "double",
			func = 	function() 
						table.foreach(FusionHUDBars, 	function(k,v)
															v.draggable = true;
															FusionHUD:LockHandler(v);
															FusionHUDModSettings.isUnlocked = true;
														end);
						FusionHUDClassBar.draggable = true;
						FusionHUD:LockHandler(FusionHUDClassBar);
						FusionHUDLockFrame:Show();
						InterfaceOptionsFrame:Hide();
					end,
		},
		mainOptions = {
			name = "Main",
			type = "group",
			order = 20,
			args = {
				transparency = {
					name = "Transparency Settings",
					type = "group",
					order = 7,
					args = {
						contextToggle = {
							name = "Context-sensitive Visibility",
							type = "toggle",
							order = 1,
							desc = "Enables changing HUD visibility based on player situation",
							descStyle = "inline",
							get = 	function() return FusionHUDModSettings.multiTransparency end,
							set =	function() FusionHUDModSettings.multiTransparency = not FusionHUDModSettings.multiTransparency end,
						},
						contextOffTransparency = {
							name = "Transparency",
							type = "range",
							desc = "Set the HUD's overall transparency",
							order = 5,
							hidden = function() return FusionHUDModSettings.multiTransparency end,
							min = 0,
							max = 100,
							step = 1,
							get = function() return FusionHUDModSettings.transparency*100 end,
							set = function(k,v) FusionHUDModSettings.transparency = v/100 end,
						},
						contextOnHiVis = {
							name = "Maximum visibility",
							type = "range",
							desc = "Set the HUD visibility while in combat",
							order = 10,
							hidden = function() return not FusionHUDModSettings.multiTransparency end,
							min = 0,
							max = 100,
							step = 1,
							get = function() return FusionHUDModSettings.hiVis*100 end,
							set = function(k,v) FusionHUDModSettings.hiVis = v/100 end,
						},
						contextOnMidVis = {
							name = "Maximum visibility",
							type = "range",
							desc = "Shows when in combat",
							order = 10,
							hidden = function() return not FusionHUDModSettings.multiTransparency end,
							min = 0,
							max = 100,
							step = 1,
							get = function() return FusionHUDModSettings.midVis*100 end,
							set = function(k,v) FusionHUDModSettings.midVis = v/100 end,
						},
						contextOnMidVis = {
							name = "Medium visibility",
							type = "range",
							desc = "Shows when: targeting, hurt, needing power and not in combat",
							order = 12,
							hidden = function() return not FusionHUDModSettings.multiTransparency end,
							min = 0,
							max = 100,
							step = 1,
							get = function() return FusionHUDModSettings.midVis*100 end,
							set = function(k,v) FusionHUDModSettings.midVis = v/100 end,
						},
						contextOnLoVis = {
							name = "Minimum visibility",
							type = "range",
							desc = "Shows when out of combat, not targeting, not debuffed, at full health/power",
							order = 14,
							hidden = function() return not FusionHUDModSettings.multiTransparency end,
							min = 0,
							max = 100,
							step = 1,
							get = function() return FusionHUDModSettings.loVis*100 end,
							set = function(k,v) FusionHUDModSettings.loVis = v/100 end,
						},
					},
				},
			},
		},
		classBar = {
			name = "Class Bar",
			type = "group",
			order = 30,
			desc = "Settings for the class-specific bar",
			args = {
				classBarToggle = {
					name = "Enable Class bar",
					type = "toggle",
					order = 1,
					desc = "Enables/Disables Class-specific bar",
					get = 	function() return FusionHUDModSettings.classBarToggle end,
					set =	function() 
								FusionHUDModSettings.classBarToggle = not FusionHUDModSettings.classBarToggle;
								if FusionHUDModSettings.classBarToggle then
									FusionHUDClassBar.frame:Show()
								else
									FusionHUDClassBar.frame:Hide()
								end
							end,
				},					
				classBarSize = {
					name = "Class bar scale",
					type = "range",
					order = 10,
					desc = "Controls the size of the class-specific bar",
					disabled = function() return not FusionHUDModSettings.classBarToggle end,
					min = 0,
					max = 150,
					step = 1,
					get = 	function() return FusionHUDClassBar.frame:GetHeight() end,
					set = 	function(k,v) 
								local i;
								FusionHUDClassBar.height = v;
								FusionHUDClassBar.frame:SetHeight(v); 
								FusionHUDClassBar.frame:SetWidth(v*6); 
								for i = 1,6 do
									FusionHUDClassBar.frame.runes[i]:SetWidth(v);
								end
							end,
				},
				classBarUnlock = {
					name = "Unlock Class bar",
					type = "execute",
					order = 10,
					disabled = function() return not FusionHUDModSettings.classBarToggle end,
					desc = "Unlocks the class bar for dragging.  Use the Lock button at the top of the screen when done.",
					func = 	function()
								FusionHUDClassBar.draggable = true;
								FusionHUD:LockHandler(FusionHUDClassBar);
								FusionHUDLockFrame:Show();
								InterfaceOptionsFrame:Hide();
							end,
				},
			},
		},		
		bars = {
			name = "Bar List",
			order = 50,
			type = "group",
			desc = "Listing of each HUD bar",
			args = {}
		},
    },
};

function FusionHUD:OnInitialize()	
	LibStub("AceConfig-3.0"):RegisterOptionsTable("FusionHUD", FusionHUDOptions);
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("FusionHUD", "FusionHUD");
	
	SLASH_FUSIONHUD1, SLASH_FUSIONHUD2 = '/fusionhud', '/fh';
end

function FusionHUD:BuildBaseFrames()
	FusionHUDMasterFrame = CreateFrame("Frame", nil, UIParent);
	FusionHUDMasterFrame:SetClampedToScreen(true);
	FusionHUDMasterFrame:SetPoint("CENTER",UIParent,"CENTER");
	FusionHUDMasterFrame:SetFrameStrata("LOW");
	FusionHUDMasterFrame:SetHeight(400);
	FusionHUDMasterFrame:SetWidth(400);
	FusionHUDMasterFrame:Show();
	FusionHUDMasterFrame:SetAlpha(1);

	FusionHUDLockFrame = CreateFrame("Frame", nil, UIParent);
	FusionHUDLockFrame:SetClampedToScreen(true);
	FusionHUDLockFrame:ClearAllPoints();
	FusionHUDLockFrame:SetPoint("TOP", UIParent, "TOP", 0, -90);
	FusionHUDLockFrame:SetFrameStrata("FULLSCREEN");
	FusionHUDLockFrame:SetHeight(100);
	FusionHUDLockFrame:SetWidth(200);
	FusionHUDLockFrame.texture = FusionHUDLockFrame:CreateTexture(nil, "BACKGROUND");
	FusionHUDLockFrame.texture:SetAllPoints(FusionHUDLockFrame);
	FusionHUDLockFrame.texture:SetTexture("Interface\\AddOns\\FusionHUD\\Graphics\\whitebackdropbase", true);
	FusionHUDLockFrame.texture:SetVertexColor(0.7, 0.1, 0.1, 1);
	FusionHUDLockFrame.texture:SetTexCoord(0,1,0,1)
	FusionHUDLockFrame.text1 = FusionHUDLockFrame:CreateFontString(nil, "HIGH")
	FusionHUDLockFrame.text1:SetPoint("TOP", FusionHUDLockFrame, "TOP",0, -7);
	FusionHUDLockFrame.text1:SetFont("Fonts\\FRIZQT__.TTF", 24, "OUTLINE");
	FusionHUDLockFrame.text1:SetJustifyH("CENTER");
	FusionHUDLockFrame.text1:SetJustifyV("TOP");
	FusionHUDLockFrame.text1:SetHeight(30);
	FusionHUDLockFrame.text1:SetWidth(150);
	FusionHUDLockFrame.text1:SetText("FusionHUD");
	FusionHUDLockFrame.text2 = FusionHUDLockFrame:CreateFontString(nil, "HIGH")
	FusionHUDLockFrame.text2:SetPoint("TOP", FusionHUDLockFrame.text1, "BOTTOM",0, 0);
	FusionHUDLockFrame.text2:SetFont("Interface\\AddOns\\FusionHUD\\Fonts\\DroidSans.ttf", 14);
	FusionHUDLockFrame.text2:SetJustifyH("CENTER");
	FusionHUDLockFrame.text2:SetJustifyV("TOP");
	FusionHUDLockFrame.text2:SetHeight(30);
	FusionHUDLockFrame.text2:SetWidth(150);
	FusionHUDLockFrame.text2:SetText("Use this button to relock your bars");
	FusionHUDLockFrame:Hide();
	
	FusionHUDLockFrame.lockButton = CreateFrame("Button", "lockButton", FusionHUDLockFrame, "UIPanelButtonTemplate")
	FusionHUDLockFrame.lockButton:SetPoint("TOP", FusionHUDLockFrame.text2, "BOTTOM", 0, 0);
	FusionHUDLockFrame.lockButton:SetWidth(100);
	FusionHUDLockFrame.lockButton:SetHeight(30);
	FusionHUDLockFrame.text1:SetVertexColor(1,1,.4,1);
	FusionHUDLockFrame.text2:SetVertexColor(1, 1, 1, 1);
	FusionHUDLockFrame.lockButton:SetText("Lock");
	FusionHUDLockFrame.lockButton:SetFrameStrata("FULLSCREEN_DIALOG");
end

function SlashCmdList.FUSIONHUD(msg, editbox)
	InterfaceOptionsFrame_OpenToCategory(FusionHUD.optionsFrame);
end

function FusionHUD:OnEnable()
	local name = UnitName("player");
	FusionHUDBars = {};
	FusionHUD:BuildBaseFrames();
	
	if FusionHUDDB then
		if FusionHUDDB[name] then
			if FusionHUDDB[name].settings then
				table.foreach(FusionHUDDB[name].settings, function(k,v)
					FusionHUDModSettings[k] = FusionHUDDB[name].settings[k]
				end);
			end
			if FusionHUDDB[name].bars then
				table.foreach(FusionHUDDB[name].bars, function(k,v)
					FusionHUDBars[k] = FusionHUDDB[name].bars[k];
					setmetatable(FusionHUDBars[k], FusionHUDBarPrototype);
					FusionHUDBars[k].lastValue = -99999999999;
					FusionHUDBars[k].lastValueMax = -99999999999;
					FusionHUDBars[k]:BuildBar();
				end);
			end
			if FusionHUDDB[name].classBar then
				table.foreach(FusionHUDDB[name].classBar, function(k,v)
					FusionHUDClassBar[k] = FusionHUDDB[name].classBar[k];
				end);
			end
		else
			FusionHUDDB[name] = {};
		end
	else
		FusionHUDDB = {};
		FusionHUDDB[name] = {};
	end
	FusionHUDDB[name].settings = FusionHUDModSettings;
	FusionHUDDB[name].bars = FusionHUDBars;
	FusionHUDDB[name].classBar = FusionHUDClassBar;	
	
	FusionHUD:BuildClassBarFrame()
	FusionHUD:ClassSelector()

	FusionHUDMasterFrame:SetScript("OnUpdate", 	function() FusionHUD:OnUpdate()	end); 	
	FusionHUDLockFrame.lockButton:SetScript('OnClick', 	function() 
		table.foreach(FusionHUDBars, function(k,v) 
			FusionHUDBars[k].draggable = false;
			FusionHUD:LockHandler(v);
			FusionHUDModSettings.isUnlocked = false;
		end);
		FusionHUDClassBar.draggable = false;
		FusionHUD:LockHandler(FusionHUDClassBar);																							
	end);
	
end	

function FusionHUD:OnUpdate()
	local FusionHUDNextUpdateClock = GetTime()
	if FusionHUDNextUpdateClock - FusionHUDLastUpdateClock > 0.1 then
		FusionHUDLastUpdateClock = FusionHUDNextUpdateClock;
		table.foreach(FusionHUDBars, 	function(k,v) 
											FusionHUDBars[k]:Update() 
										end);
	end
end

function FusionHUD:MinimapButtonOnEnter()
	GameTooltip:SetOwner(FusionHUDMinimapButtonFrame, "ANCHOR_TOPRIGHT");
	GameTooltip:ClearLines();
	GameTooltip:SetAlpha(1);
	if FusionHUDLockFrame:IsVisible() then
		GameTooltip:SetText("FusionHUD |cffff9933" .. GetAddOnMetadata("FusionHUD", "Version") .. "|r", 1, .9, 0,  1, 1);
		GameTooltip:AddLine("|cffff3333Please relock your bars using the FusionHUD Lock Button");
	else
		GameTooltip:SetText("FusionHUD |cffff9933" .. GetAddOnMetadata("FusionHUD", "Version") .. "|r", 1, .9, 0,  1, 1);
		GameTooltip:AddLine("|cff44ff44Click to open FusionHUD Options", 1,1,1,1,1);
	end
	GameTooltip:Show();
end

function FusionHUD:MinimapButtonOnLeave(...)
	GameTooltip:Hide();
end

function FusionHUD:DeleteBar()
	local index = -1;
	table.foreach(FusionHUDBars, function(k,v) if index < v.barIndex then index = v.barIndex end end);
	if index then
		FusionHUDOptions.args.bars.args["bar" .. index] = nil;
		FusionHUDBars["bar" .. index].frame:Hide();
		FusionHUDBars["bar" .. index] = nil;
	else
		FusionHUDBars = {};
	end
end

function FusionHUD:Transparency()
	local localClass, englishClass = UnitClass("player");
	if FusionHUDModSettings.multiTransparency == false then
		FusionHUDMasterFrame:SetAlpha(FusionHUDModSettings.transparency);
	elseif UnitAffectingCombat("player") == 1 then
		FusionHUDMasterFrame:SetAlpha(FusionHUDModSettings.hiVis);
	elseif UnitDebuff("player", 1) ~= nil then
		FusionHUDMasterFrame:SetAlpha(FusionHUDModSettings.midVis);
	elseif UnitName("target") and UnitName("target") ~= "Unknown" then
		FusionHUDMasterFrame:SetAlpha(FusionHUDModSettings.midVis);
	elseif UnitHealth("player") ~= UnitHealthMax("player") then
		FusionHUDMasterFrame:SetAlpha(FusionHUDModSettings.midVis);
	elseif UnitPower("player")~= UnitPowerMax("player") then
		if englishClass == "WARRIOR" or englishClass == "DEATHKNIGHT" then
			if UnitPower("player") == 0 then
				FusionHUDMasterFrame:SetAlpha(FusionHUDModSettings.loVis);
			else
				FusionHUDMasterFrame:SetAlpha(FusionHUDModSettings.midVis);
			end
		else
			FusionHUDMasterFrame:SetAlpha(FusionHUDModSettings.midVis);
		end
	else
		FusionHUDMasterFrame:SetAlpha(FusionHUDModSettings.loVis);
	end
end

function FusionHUD:BuildClassBarFrame()
	FusionHUDClassBar.frame = CreateFrame("Frame", nil, FusionHUDMasterFrame);
	FusionHUDClassBar.frame:SetPoint(FusionHUDClassBar.myPoint, "FusionHUDMasterFrame", FusionHUDClassBar.parentPoint, FusionHUDClassBar.offsetX, FusionHUDClassBar.offsetY);
	FusionHUDClassBar.frame:SetHeight(FusionHUDClassBar.height);
	FusionHUDClassBar.frame:SetWidth(FusionHUDClassBar.height * 6);
	FusionHUDClassBar.frame.backdropTexture = FusionHUDClassBar.frame:CreateTexture(nil, "BACKGROUND");
	FusionHUDClassBar.frame.backdropTexture:SetTexture("Interface\\AddOns\\FusionHUD\\Graphics\\whitebackdropbase", true);
	FusionHUDClassBar.frame.backdropTexture:ClearAllPoints();
	FusionHUDClassBar.frame.titleRegion = FusionHUDClassBar.frame:CreateTitleRegion();
	FusionHUDClassBar.frame.titleRegion:ClearAllPoints();
	if FusionHUDModSettings.classBarToggle then
		FusionHUDClassBar.frame:Show();
	else
		FusionHUDClassBar.frame:Hide();
	end
end

function FusionHUD:ClassSelector()
	local localizedClass, englishClass = UnitClass("player");
	if englishClass == "DEATHKNIGHT" then
		FusionHUD:BuildDKClassBar()
	end
end

function FusionHUD:BuildDKClassBar()
	local i;
	FusionHUDClassBar.frame.runes = {};
	for i = 1,6 do
		FusionHUDClassBar.frame.runes[i] = FusionHUDRuneObject:New();
		FusionHUDClassBar.frame.runes[i].runeID = i;
		FusionHUDClassBar.frame.runes[i].texture:SetTexture(runeTextures[GetRuneType(i)]);
	end
	for i = 1,6 do
		if i == 1 then
			FusionHUDClassBar.frame.runes[i]:SetPoint("TOPLEFT", FusionHUDClassBar.frame, "TOPLEFT");
			FusionHUDClassBar.frame.runes[i]:SetPoint("BOTTOMLEFT", FusionHUDClassBar.frame, "BOTTOMLEFT");
		else
			FusionHUDClassBar.frame.runes[i]:SetPoint("TOPLEFT", FusionHUDClassBar.frame.runes[i-1], "TOPRIGHT");
			FusionHUDClassBar.frame.runes[i]:SetPoint("BOTTOMLEFT", FusionHUDClassBar.frame.runes[i-1], "BOTTOMRIGHT");
		end
	end
	FusionHUD:RegisterEvent("RUNE_POWER_UPDATE");
	FusionHUD:RegisterEvent("RUNE_TYPE_UPDATE");
end

function FusionHUD:RUNE_POWER_UPDATE(event, ...)
	local runeID = ...;
	local start, duration, runeReady = GetRuneCooldown(runeID);
	if runeReady == false then
		FusionHUDClassBar.frame.runes[runeID].texture:SetVertexColor(0.1,0.1,0.1);
		FusionHUDClassBar.frame.runes[runeID].cooldownFrame:Show();
		FusionHUDClassBar.frame.runes[runeID].cooldownFrame:SetScript('OnUpdate', function() FusionHUD:RuneCooldown(runeID) end);
	else
		FusionHUDClassBar.frame.runes[runeID].texture:SetVertexColor(1,1,1);
		FusionHUDClassBar.frame.runes[runeID].cooldownFrame:SetScript('OnUpdate', nil);
		FusionHUDClassBar.frame.runes[runeID].cooldownFrame:Hide();
	end
end

function FusionHUD:RUNE_TYPE_UPDATE(event, ...)
	local runeID = ...;
	local runeType = GetRuneType(runeID);
	FusionHUDClassBar.frame.runes[runeID].texture:SetTexture(runeTextures[runeType], false);
end

function FusionHUD:RuneCooldown(runeID)
	local nextUpdateClock =  GetTime();
	local start, duration, runeReady = GetRuneCooldown(runeID);
	if nextUpdateClock - FusionHUDClassBar.frame.runes[runeID].cooldownFrame.lastUpdateClock > 0.2 then
		FusionHUDClassBar.frame.runes[runeID].cooldownFrame.lastUpdateClock = nextUpdateClock
		if runeReady == false and math.ceil(duration - (nextUpdateClock - start)) >= 0 and math.ceil(duration - (nextUpdateClock - start)) <= duration then
			FusionHUDClassBar.frame.runes[runeID].cooldownFrame.text:SetFont("Interface\\AddOns\\FusionHUD\\Fonts\\DroidSans.ttf", math.floor(FusionHUDClassBar.frame:GetHeight() * .6), "OUTLINE");
			FusionHUDClassBar.frame.runes[runeID].cooldownFrame.text:SetText(math.ceil(duration-(nextUpdateClock-start)));
			FusionHUDClassBar.frame.runes[runeID].cooldownFrame.text:SetVertexColor(1,1,1,1);
		else
			FusionHUDClassBar.frame.runes[runeID].cooldownFrame.text:SetVertexColor(1,1,1,0);
		end
	end
end

function FusionHUD:LockHandler(self)
	if self.draggable == true then
		self.frame.titleRegion:SetAllPoints(self.frame);
		self.frame:SetBackdrop({bgFile = "Interface\\AddOns\\FusionHUD\\Graphics\\whitebackdropbase", tile = true, tilesize = self.width});
		self.frame:SetBackdropColor(0.8, 0.8, 0.8, 1);
		self.frame:Show();
		self.frame:SetAlpha(1);
		self.frame:EnableMouse(true);
		self.frame:SetMovable(true);
		return
	else
		self.myPoint, self.parentFrame, self.parentPoint, self.offsetX, self.offsetY = self.frame:GetPoint();
		self.frame.titleRegion:ClearAllPoints();
		self.frame:SetBackdrop({nil});
		self.frame:EnableMouse(false);
		self.frame:SetMovable(false);
		if FusionHUDLockFrame:IsShown() then
			FusionHUDLockFrame:Hide();
			InterfaceOptionsFrame:Show();
		end
	end
end